@@ -57,10 +57,18 @@ use vars qw( %CONSTANTS );
     'SSH2_MSG_DEBUG' => 4,
     'SSH2_MSG_SERVICE_REQUEST' => 5,
     'SSH2_MSG_SERVICE_ACCEPT' => 6,
+    'SSH2_MSG_EXT_INFO' => 7,
     'SSH2_MSG_KEXINIT' => 20,
     'SSH2_MSG_NEWKEYS' => 21,
     'SSH2_MSG_KEXDH_INIT' => 30,
     'SSH2_MSG_KEXDH_REPLY' => 31,
+    'SSH2_MSG_KEX_DH_GEX_REQUEST_OLD' => 30,
+    'SSH2_MSG_KEX_DH_GEX_REQUEST' => 34,
+    'SSH2_MSG_KEX_DH_GEX_GROUP' => 31,
+    'SSH2_MSG_KEX_DH_GEX_INIT' => 32,
+    'SSH2_MSG_KEX_DH_GEX_REPLY' => 33,
+    'SSH2_MSG_KEX_ECDH_INIT'  => 30,
+    'SSH2_MSG_KEX_ECDH_REPLY' => 31,
     'SSH2_MSG_USERAUTH_REQUEST' => 50,
     'SSH2_MSG_USERAUTH_FAILURE' => 51,
     'SSH2_MSG_USERAUTH_SUCCESS' => 52,
@@ -68,6 +76,9 @@ use vars qw( %CONSTANTS );
     'SSH2_MSG_USERAUTH_PK_OK' => 60,
     'SSH2_MSG_USERAUTH_INFO_REQUEST' => 60,
     'SSH2_MSG_USERAUTH_INFO_RESPONSE' => 61,
+    'SSH2_MSG_GLOBAL_REQUEST' => 80,
+    'SSH2_MSG_REQUEST_SUCCESS' => 81,
+    'SSH2_MSG_REQUEST_FAILURE' => 82,
     'SSH2_MSG_CHANNEL_OPEN' => 90,
     'SSH2_MSG_CHANNEL_OPEN_CONFIRMATION' => 91,
     'SSH2_MSG_CHANNEL_OPEN_FAILURE' => 92,
@@ -112,11 +123,17 @@ use vars qw( %CONSTANTS );
     'CHAN_CLOSE_RCVD' => 0x02,
 
     'KEX_DH1' => 'diffie-hellman-group1-sha1',
-    'KEX_DH14' => 'diffie-hellman-group14-sha1',
-    'KEX_DEFAULT_KEX' => 'diffie-hellman-group1-sha1,diffie-hellman-group14-sha1',
-    'KEX_DEFAULT_PK_ALG' => 'ssh-dss,ssh-rsa',
-    'KEX_DEFAULT_ENCRYPT' => '3des-cbc,blowfish-cbc,arcfour',
-    'KEX_DEFAULT_MAC' => 'hmac-sha1,hmac-md5',
+    'KEX_DH14_SHA1' => 'diffie-hellman-group14-sha1',
+    'KEX_DH14_SHA256' => 'diffie-hellman-group14-sha256',
+    'KEX_DH16_SHA512' => 'diffie-hellman-group16-sha512',
+    'KEX_DH18_SHA512' => 'diffie-hellman-group18-sha512',
+    'KEX_DH_GEX_SHA256' => 'diffie-hellman-group-exchange-sha256',
+    'KEX_DH_GEX_SHA1' => 'diffie-hellman-group-exchange-sha1',
+    'KEX_CURVE25519_SHA256' => 'curve25519-sha256@libssh.org',
+    'KEX_DEFAULT_KEX' => 'curve25519-sha256@libssh.org,diffie-hellman-group16-sha512,diffie-hellman-group18-sha512,diffie-hellman-group-exchange-sha256,diffie-hellman-group14-sha256,diffie-hellman-group14-sha1',
+    'KEX_DEFAULT_PK_ALG' => 'ssh-ed25519,rsa-sha2-512,rsa-sha2-256,ecdsa-sha2-nistp521,ecdsa-sha2-nistp384,ecdsa-sha2-nistp256,ssh-rsa',
+    'KEX_DEFAULT_ENCRYPT' => 'chacha20-poly1305@openssh.com,aes256-ctr,aes192-ctr,aes128-ctr',
+    'KEX_DEFAULT_MAC' => 'hmac-sha2-512-etm@openssh.com,hmac-sha2-256-etm@openssh.com,hmac-sha2-512,hmac-sha2-256',
     'KEX_DEFAULT_COMP' => 'none,zlib',
     'KEX_DEFAULT_LANG' => '',
 
